@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2003-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,29 +23,11 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 2003-2004 Apple Computer, Inc.  All rights reserved.
  *
  *
  */
-//		$Log: IOPlatformPWMFanControl.h,v $
-//		Revision 1.3  2003/06/07 01:30:56  eem
-//		Merge of EEM-PM72-ActiveFans-2 branch, with a few extra tweaks.  This
-//		checkin has working PID control for PowerMac7,2 platforms, as well as
-//		a first shot at localized strings.
-//		
-//		Revision 1.2.2.1  2003/05/26 10:07:14  eem
-//		Fixed most of the bugs after the last cleanup/reorg.
-//		
-//		Revision 1.2  2003/05/21 21:58:49  eem
-//		Merge from EEM-PM72-ActiveFans-1 branch with initial crack at active fan
-//		control on Q37.
-//		
-//		Revision 1.1.2.1  2003/05/17 11:08:22  eem
-//		All active fan data present, table event-driven.  PCI power sensors are
-//		not working yet so PCI fan is just set to 67% PWM and forgotten about.
-//		
-//
-//
+
 
 #ifndef _IOPLATFORMPWMFANCONTROL_H
 #define _IOPLATFORMPWMFANCONTROL_H
@@ -63,8 +45,8 @@ protected:
 
 	// the FCU represents PWM as a value in the range [0..255] for ticks per cycle
 	// we need to convert that to a value in the range [0..100] for duty cycle %
-	virtual const OSNumber * applyTargetValueTransform( const OSNumber * hwReading );
-	virtual const OSNumber * applyTargetHWTransform( const OSNumber * value );
+	virtual ControlValue			applyTargetValueTransform( ControlValue hwReading );
+	virtual ControlValue			applyTargetValueInverseTransform( ControlValue pluginReading );
 
 public:
 
@@ -72,4 +54,4 @@ public:
 
 };
 
-#endif // _IOPLATFORMPWMFANCONTROL_H
\ No newline at end of file
+#endif // _IOPLATFORMPWMFANCONTROL_H
