@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,82 +23,19 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc.  All rights reserved.
  *
  *  DRI: Dave Radcliffe
  *
  */
-//		$Log: IOPlatformSensor.h,v $
-//		Revision 1.7  2003/07/08 04:32:49  eem
-//		3288891, 3279902, 3291553, 3154014
-//		
-//		Revision 1.6  2003/06/25 02:16:24  eem
-//		Merged 101.0.21 to TOT, fixed PM72 lproj, included new fan settings, bumped
-//		version to 101.0.22.
-//		
-//		Revision 1.5.4.2  2003/06/20 09:07:33  eem
-//		Added rising/falling slew limiters, integral clipping, etc.
-//		
-//		Revision 1.5.4.1  2003/06/19 10:24:16  eem
-//		Pulled common PID code into IOPlatformPIDCtrlLoop and subclassed it with
-//		PowerMac7_2_CPUFanCtrlLoop and PowerMac7_2_PIDCtrlLoop.  Added history
-//		length to meta-state.  No longer adjust T_err when the setpoint changes.
-//		Don't crank the CPU fans for overtemp, just slew slow.
-//		
-//		Revision 1.5  2003/06/07 01:30:56  eem
-//		Merge of EEM-PM72-ActiveFans-2 branch, with a few extra tweaks.  This
-//		checkin has working PID control for PowerMac7,2 platforms, as well as
-//		a first shot at localized strings.
-//		
-//		Revision 1.4.2.3  2003/06/06 12:16:49  eem
-//		Updated strings, turned off debugging.
-//		
-//		Revision 1.4.2.2  2003/05/23 05:44:40  eem
-//		Cleanup, ctrlloops not get notification for sensor and control registration.
-//		
-//		Revision 1.4.2.1  2003/05/22 01:31:04  eem
-//		Checkin of today's work (fails compilations right now).
-//		
-//		Revision 1.4  2003/05/21 21:58:49  eem
-//		Merge from EEM-PM72-ActiveFans-1 branch with initial crack at active fan
-//		control on Q37.
-//		
-//		Revision 1.3.2.3  2003/05/17 11:08:22  eem
-//		All active fan data present, table event-driven.  PCI power sensors are
-//		not working yet so PCI fan is just set to 67% PWM and forgotten about.
-//		
-//		Revision 1.3.2.2  2003/05/16 07:08:45  eem
-//		Table-lookup active fan control working with this checkin.
-//		
-//		Revision 1.3.2.1  2003/05/14 22:07:49  eem
-//		Implemented state-driven sensor, cleaned up "const" usage and header
-//		inclusions.
-//		
-//		Revision 1.3  2003/05/13 02:13:51  eem
-//		PowerMac7_2 Dynamic Power Step support.
-//		
-//		Revision 1.2.2.1  2003/05/12 11:21:10  eem
-//		Support for slewing.
-//		
-//		Revision 1.2  2003/05/10 06:50:33  eem
-//		All sensor functionality included for PowerMac7_2_PlatformPlugin.  Version
-//		is 1.0.1d12.
-//		
-//		Revision 1.1.1.1.2.2  2003/05/03 01:11:38  eem
-//		*** empty log message ***
-//		
-//		Revision 1.1.1.1.2.1  2003/05/01 09:28:40  eem
-//		Initial check-in in progress toward first Q37 checkpoint.
-//		
-//		Revision 1.1.1.1  2003/02/04 00:36:43  raddog
-//		initial import into CVS
-//		
+
 
 #ifndef _IOPLATFORMSENSOR_H
 #define _IOPLATFORMSENSOR_H
 
 // IOService pulls in all the headers we need
 #include <IOKit/IOService.h>
+#include "IOPlatformPluginDefs.h"
 
 #ifdef SENSOR_DLOG
 #undef SENSOR_DLOG
@@ -148,23 +85,89 @@ protected:
 
 	// sometimes we need to apply a transform to convert a value read from the sensor into something meaningful.
 	// Subclasses should implement this method to perform their transforms.
-	virtual const OSNumber *	applyValueTransform( const OSNumber * hwReading ) const;
+	//
+	// This method operates on the OSNumber object that's passed into it by using the OSNumber::setValue()
+	// method.  This should NOT be used on the object that belongs to the IOHWSensor instance, rather it
+	// should be used on the platform plugin's object.
+	virtual SensorValue applyCurrentValueTransform( SensorValue ) const;
 
 	// notify the control loops that we've got a registered driver
 	virtual void notifyCtrlLoops( void );
 
 public:
-	// initialize a sensor from it's SensorArray dict in the IOPlatformThermalProfile
+
+	/*
+	 * Notes on initPlatformSensor( const OSDictionary * ), initPlatformSensor( IOService * ),
+	 * and registerDriver( IOSerivce *, const OSDictionary *, bool )
+	 *
+	 * These routines are cumulatively responsible for initializing an IOPlatformSensor object
+	 * and making sure that all the correct data is supplied, parsed, and prioritized.  There
+	 * are a number of properties inside a sensor.  Some can be declared by the thermal
+	 * profile, some can be supplied by the registering driver, and some, if left unspecified,
+	 * will be set to some default value.
+	 *
+	 * initPlatformSensor( const OSDictionary * ) is invoked when a sensor is being initialized
+	 * from a thermal profile entry.  The OSDictionary parameter is the thermal profile entry.
+	 *
+	 * initPlatformSensor( IOService * ) is invoked when a sensor driver tries to register and
+	 * its sensor-id is not found in the thermal profile.  The IOService parameter is the driver
+	 * that is sending the registration.
+	 *
+	 * registerDriver( IOService *, const OSDictionary *, bool ) is invoked after one of the
+	 * initPlatformSensor() routines has been invoked (and some of the parameters have already
+	 * been set.
+	 *
+	 * The following properties are REQUIRED for successful completion of initPlatformSensor:
+	 *	kIOPPluginSensorIDKey
+	 *
+	 * The following properties may be supplied by the thermal profile, but will be given
+	 * default values if not specified (the sensor driver does not know about these
+	 * properties):
+	 *
+	 *	kIOPPluginThermalLocalizedDescKey		"UNKNOWN_SENSOR"	<OSString>
+	 *	kIOPPluginSensorFlagsKey				0x0					<OSNumber>
+	 *
+	 * The following properties are always supplied by the sensor driver (IOHWSensor) but, if
+	 * specified, values in the thermal profile will take precedence and be used instead.
+	 *
+	 *	kIOPPluginTypeKey
+	 *	kIOPPluginVersionKey
+	 *	kIOPPluginLocationKey
+	 *	kIOPPluginZoneKey
+	 *
+	 * The following parameters may not be supplied at all, by either the thermal profile
+	 * or by the sensor driver.  The value will be searched for with the highest precedence
+	 * on the thermal profile, then next on the sensor driver.  If not specified in either
+	 * place, a default will be assigned.
+	 *
+	 *	kIOPPluginPollingPeriodKey				-1					<OSNumber>
+	 *
+	 */
+
+	/*! @function initPlatformSensor
+		@abstract Initialize an IOPlatformSensor object from a thermal profile entry
+		@param dict An element from the SensorArray array of the thermal profile
+	*/
 	virtual IOReturn	initPlatformSensor( const OSDictionary * dict );
 
-	// initialize an IOPlatformSensor from an unknown (i.e. not listed in thermal profile) sensor
-	// that is trying to register.
-	virtual IOReturn	initPlatformSensor( IOService * unknownSensor );
+	/*! @function initPlatformSensor
+		@abstract Initialize an IOPlatformSensor object from an unknown (i.e. not listed in thermal profile) sensor that is trying to register
+		@param unknownSensor An IOService reference to the sender of the registration request
+		@param dict The OSDictionary supplied with the registration message
+	*/
+	virtual IOReturn	initPlatformSensor( IOService * unknownSensor, const OSDictionary * dict );
 
-	/*! @function isRegistered Tells whether there is an IOService * associated with this sensor. */
+	/*! @function isRegistered
+		@abstract Tells whether there is an IOService * associated with this sensor.
+	*/
 	virtual OSBoolean *	isRegistered(void);
 
-	/*!	@function registerDriver Used to associated an IOService with this sensor. */
+	/*!	@function registerDriver
+		@abstract Used to associated an IOService with this sensor, responsible for extracting needed data from the sensor driver instance and sending the polling period to the sensor driver
+		@param driver An IOService reference to the sender of the registration request
+		@param dict The registration dictionary send by the driver
+		@param notify A boolean that can be used by IOPlatformSensor subclasses that can be used to properly synchronize notification to IOPlatformCtrlLoops that this sensor has registered.  See IOPlatformStateSensor for example usage.
+	*/
 	virtual IOReturn	registerDriver(IOService *driver, const OSDictionary * dict, bool notify = true);
 
 	/*! @function joinedCtrlLoop
@@ -212,14 +215,25 @@ public:
 	virtual const OSNumber *		getSensorFlags( void );
 
 	// current-value
-	virtual const OSNumber *		getCurrentValue( void );
-	virtual void					setCurrentValue( const OSNumber * sensorValue );
+	virtual SensorValue				getCurrentValue( void );
+	virtual void					setCurrentValue( SensorValue newValue );
 
-	// this sends a force-update message to the sensor and then reads its current-value property
-	virtual const OSNumber *		fetchCurrentValue( void );	// blocks for I/O
+	/*! @function fetchCurrentValue
+		@abstract Read the "current-value" currently stored in the IOHWSensor instance that corresponds to this sensor and return the transformed (if applicable) fresh value.  This does NOT force IOHWSensor to poll the hardware before utilizing the current-value it's published.
+	*/
+	virtual SensorValue				fetchCurrentValue( void );	// blocks for I/O
+
+	/*!	@function sendForceUpdateMessage
+		@abstract Formulates and sends a force-update message to the sensor.  This causes the IOHWSensor driver to poll the hardware and update it's current-value. */
+	virtual IOReturn				sendForceUpdate( void );
+
+	/*! @function forceAndFetchCurrentValue
+		@abstract Same as fetchCurrentValue(), except calls sendForceUpdate() before fetching, transforming and storing the current-value.
+	*/
+	virtual SensorValue				forceAndFetchCurrentValue( void );	// blocks for I/O
 
 	// polling period
-	virtual const OSNumber *		getPollingPeriod( void );
+	virtual OSNumber *				getPollingPeriod( void );
 	virtual void					setPollingPeriod( const OSNumber * period );
 
 	// this sends the polling period to the sensor
@@ -240,4 +254,4 @@ public:
 };
 
 
-#endif 	// _IOPLATFORMSENSOR_H
\ No newline at end of file
+#endif 	// _IOPLATFORMSENSOR_H
