@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,52 +23,11 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc.  All rights reserved.
  *
  *
  */
-//		$Log: IOPlatformTableCtrlLoop.cpp,v $
-//		Revision 1.5  2003/07/16 02:02:09  eem
-//		3288772, 3321372, 3328661
-//		
-//		Revision 1.4  2003/07/08 04:32:49  eem
-//		3288891, 3279902, 3291553, 3154014
-//		
-//		Revision 1.3  2003/06/07 01:30:56  eem
-//		Merge of EEM-PM72-ActiveFans-2 branch, with a few extra tweaks.  This
-//		checkin has working PID control for PowerMac7,2 platforms, as well as
-//		a first shot at localized strings.
-//		
-//		Revision 1.2.2.6  2003/06/06 08:17:56  eem
-//		Holy Motherfucking shit.  PID is really working.
-//		
-//		Revision 1.2.2.5  2003/06/01 14:52:51  eem
-//		Most of the PID algorithm is implemented.
-//		
-//		Revision 1.2.2.4  2003/05/29 03:51:34  eem
-//		Clean up environment dictionary access.
-//		
-//		Revision 1.2.2.3  2003/05/26 10:07:15  eem
-//		Fixed most of the bugs after the last cleanup/reorg.
-//		
-//		Revision 1.2.2.2  2003/05/23 06:36:57  eem
-//		More registration notification stuff.
-//		
-//		Revision 1.2.2.1  2003/05/22 01:31:04  eem
-//		Checkin of today's work (fails compilations right now).
-//		
-//		Revision 1.2  2003/05/21 21:58:49  eem
-//		Merge from EEM-PM72-ActiveFans-1 branch with initial crack at active fan
-//		control on Q37.
-//		
-//		Revision 1.1.2.2  2003/05/17 12:55:37  eem
-//		Active fan control works on RPM channels!!!!!!
-//		
-//		Revision 1.1.2.1  2003/05/16 07:08:45  eem
-//		Table-lookup active fan control working with this checkin.
-//		
-//
-//
+
 
 #include "IOPlatformPlugin.h"
 #include "IOPlatformPluginDefs.h"
@@ -213,7 +172,8 @@ bool IOPlatformTableCtrlLoop::updateMetaState( void )
 
 void IOPlatformTableCtrlLoop::adjustControls( void )
 {
-	const OSNumber * state, * newTarget;
+	const OSNumber * state, * newTargetNum;
+	ControlValue newTarget;
 
 	if (ctrlloopState == kIOPCtrlLoopNotReady)
 	{
@@ -228,16 +188,18 @@ void IOPlatformTableCtrlLoop::adjustControls( void )
 		return;
 	}
 
-	if ((newTarget = OSDynamicCast(OSNumber, lookupTable->getObject( state->unsigned32BitValue() ))) == NULL)
+	if ((newTargetNum = OSDynamicCast(OSNumber, lookupTable->getObject( state->unsigned32BitValue() ))) == NULL)
 	{
 		CTRLLOOP_DLOG("IOPlatformTableCtrlLoop::adjustControls no target for state %u\n", state->unsigned32BitValue() );
 		return;
 	}
 
+	newTarget = newTargetNum->unsigned32BitValue();
+
 	// If the new target value is different, send it to the control
 	if (ctrlloopState == kIOPCtrlLoopFirstAdjustment ||
 	    ctrlloopState == kIOPCtrlLoopDidWake ||
-	    !newTarget->isEqualTo( outputControl->getTargetValue() ))
+	    newTarget != outputControl->getTargetValue() )
 	{
 		if (outputControl->sendTargetValue( newTarget ))
 		{
@@ -269,4 +231,4 @@ void IOPlatformTableCtrlLoop::controlRegistered( IOPlatformControl * aControl )
 		ctrlloopState = kIOPCtrlLoopFirstAdjustment;
 		adjustControls();
 	}
-}
\ No newline at end of file
+}
