@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
@@ -23,38 +23,12 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 /*
- * Copyright (c) 2002-2003 Apple Computer, Inc.  All rights reserved.
+ * Copyright (c) 2002-2004 Apple Computer, Inc.  All rights reserved.
  *
  *  DRI: Dave Radcliffe
  *
  */
-//		$Log: MacRISC4CPU.h,v $
-//		Revision 1.8  2003/06/03 23:03:57  raddog
-//		disable second cpu when unused - 3249029, 3273619
-//		
-//		Revision 1.7  2003/05/07 00:14:55  raddog
-//		[3125575] MacRISC4 initial sleep support
-//		
-//		Revision 1.6  2003/04/27 23:13:30  raddog
-//		MacRISC4PE.cpp
-//		
-//		Revision 1.5  2003/03/04 17:53:20  raddog
-//		[3187811] P76: U3.2.0 systems don't boot
-//		[3187813] MacRISC4CPU bridge saving code can block on interrupt stack
-//		[3138343] Q37 Feature: remove platform functions for U3
-//		
-//		Revision 1.4  2003/02/27 01:42:54  raddog
-//		Better support for MP across sleep/wake [3146943]. This time we block in startCPU, rather than initCPU, which is safer.
-//		
-//		Revision 1.3  2003/02/19 21:54:45  raddog
-//		Support for MP across sleep/wake [3146943]
-//		
-//		Revision 1.2  2003/02/18 00:02:01  eem
-//		3146943: timebase enable for MP, bump version to 1.0.1d3.
-//		
-//		Revision 1.1.1.1  2003/02/04 00:36:43  raddog
-//		initial import into CVS
-//		
+
 
 #ifndef _IOKIT_MACRISC4CPU_H
 #define _IOKIT_MACRISC4CPU_H
@@ -67,6 +41,18 @@
 class MacRISC4PE;
 class MacRISC4CPUInterruptController;
 
+// data structure to hold platform-specific timebase sync parameters
+typedef struct
+{
+	const char * i2c_iface;		// string identifier for desired PPCI2CInterface in IOResources
+	UInt8 i2c_port;				// i2c port 0 or 1
+	UInt8 i2c_addr;				// 8-bit i2c slave address
+	UInt8 i2c_subaddr;			// 8-bit i2c register subaddress
+	UInt8 mask;					// mask to be applied during writes to device register
+	UInt8 enable_value;			// value for enabling timebase clocks
+	UInt8 disable_value;		// value for stopping timebase clocks
+} cpu_timebase_params_t;
+
 enum {
 	kMaxPCIBridges = 32
 };
@@ -95,8 +81,9 @@ private:
     UInt32				currentProcessorSpeed;
 	UInt32				topLevelPCIBridgeCount;
 	IOPCIBridge			*topLevelPCIBridges[kMaxPCIBridges];
-    
-    virtual void ipiHandler(void *refCon, void *nub, int source);
+
+	static	void		sIPIHandler( OSObject* self, void* refCon, IOService* nub, int source );
+	virtual	void		ipiHandler(void *refCon, void *nub, int source);
 
     // callPlatformFunction symbols
     const OSSymbol 		*mpic_dispatchIPI;
@@ -120,6 +107,8 @@ private:
     const OSSymbol 		*i2c_writeI2CBus;
     const OSSymbol 		*u3APIPhyDisableProcessor1;
 
+	static	void			sEnableCPUTimeBase( cpu_id_t self, boolean_t enable );
+
 public:
     virtual const OSSymbol *getCPUName(void);
   
