@@ -28,45 +28,7 @@
  *  DRI: Dave Radcliffe
  *
  */
-//		$Log: U3.cpp,v $
-//		Revision 1.11.8.1  2003/09/09 20:23:37  raddog
-//		[3398022] Update sleep code for case where golem isn't present (Q37A)
-//		
-//		Revision 1.11  2003/07/24 21:15:56  raddog
-//		[3336924]Do not reset MPIC across sleep - fixes DVD playback after sleep
-//		
-//		Revision 1.10  2003/07/03 01:16:32  raddog
-//		[3313953]U3 PwrMgmt register workaround
-//		
-//		Revision 1.9  2003/06/27 00:45:07  raddog
-//		[3304596]: remove unnecessary access to U3 Pwr registers on wake, [3249029]: Disable unused second process on wake, [3301232]: remove unnecessary PCI code from PE
-//		
-//		Revision 1.8  2003/06/03 23:03:57  raddog
-//		disable second cpu when unused - 3249029, 3273619
-//		
-//		Revision 1.7  2003/06/03 01:50:24  raddog
-//		U3 sleep changes including calling SPU
-//		
-//		Revision 1.6  2003/05/07 00:14:55  raddog
-//		[3125575] MacRISC4 initial sleep support
-//		
-//		Revision 1.5  2003/04/04 01:27:27  raddog
-//		[3217587]: Q37: AppleU3 needs to enable U3 MPIC
-//		
-//		Revision 1.4  2003/03/04 17:53:20  raddog
-//		[3187811] P76: U3.2.0 systems don't boot
-//		[3187813] MacRISC4CPU bridge saving code can block on interrupt stack
-//		[3138343] Q37 Feature: remove platform functions for U3
-//		
-//		Revision 1.3  2003/02/27 01:42:54  raddog
-//		Better support for MP across sleep/wake [3146943]. This time we block in startCPU, rather than initCPU, which is safer.
-//		
-//		Revision 1.2  2003/02/18 00:02:01  eem
-//		3146943: timebase enable for MP, bump version to 1.0.1d3.
-//		
-//		Revision 1.1.1.1  2003/02/04 00:36:43  raddog
-//		initial import into CVS
-//		
+
 
 #include <IOKit/platform/ApplePlatformExpert.h>
 
@@ -152,7 +114,7 @@ bool AppleU3::start ( IOService * nub )
     symSetSPUSleep = OSSymbol::withCString("setSPUsleep");
     symSetPMUSleep = OSSymbol::withCString("sleepNow");
 	symU3APIPhyDisableProcessor1 = OSSymbol::withCString("u3APIPhyDisableProcessor1");
-    
+
 	// Identify any platform-do-functions
 	retval = callPlatformFunction (functionSymbol, true, (void *)provider, 
 		(void *)&platformFuncArray, (void *)0, (void *)0);
@@ -165,20 +127,31 @@ bool AppleU3::start ( IOService * nub )
 				if (func->getCommandFlags() & kIOPFFlagOnDemand)
 					func->publishPlatformFunction (this);
 	}
-	
+
 	// If we have our own MPIC, enable it
 	if (mpicRegEntry = fromPath ("mpic", gIODTPlane, NULL, NULL, provider)) {
 		// Set interrupt enable bits in U3 toggle register
 		safeWriteRegUInt32(kU3ToggleRegister, kU3MPICEnableOutputs | kU3MPICReset, 
 			kU3MPICEnableOutputs | kU3MPICReset);
 	}
-		
+
 	// Create our friends
 	createNubs(this, provider->getChildIterator( gIODTPlane ));
   	
 	// Come and get it...
 	registerService();
-	
+
+	// If we have a chip fault interrupt, install the handler and enable notifications.  It's normal for
+	// this to fail on some platforms, since U3.1 and U3.2 don't have a chip fault pin.
+	//
+	// this must come after registerService() because it does a waitForService() ...
+	if (kIOReturnSuccess == installChipFaultHandler( provider ))
+	{
+		// we have a chip fault notifier, initialize related functionality such as ECC
+		setupDARTExcp();
+		setupECC();
+	}
+
 	return super::start(provider);
 }
 
@@ -193,6 +166,12 @@ void AppleU3::free ()
 	if (mutex != NULL)
 		IOSimpleLockFree( mutex );
 
+	if (dimmErrors)
+		IOFree( dimmErrors, sizeof(u3_parity_error_record_t) * dimmCount );
+
+	if (dimmLock)
+		IOSimpleLockFree( dimmLock );
+
 	super::free();
 	
 	return;
@@ -296,20 +275,6 @@ IOReturn AppleU3::callPlatformFunction(const char *functionName, bool waitForFun
 // **********************************************************************************
 UInt32 AppleU3::readUniNReg(UInt32 offset)
 {
-	// [3313953] reads to U3 Pwr Mgmt registers can sometimes return bogus results
-	// The fix is to keep reading until a consistent result is obtained.
-	if ((offset >= kU3PMClockControl) && (offset <= kU3PMSMax)) {
-		UInt32 result1, result2;
-		
-		result2 = uniNBaseAddress[offset >> 2];			// Read it
-		do {
-			result1 = result2;							// Save latest result
-			result2 = uniNBaseAddress[offset >> 2];		// Read it again
-		} while (result1 != result2);					// Until result is consistent
-		return result2;
-	}
-	
-	// Normal case - no workaround necessary
     return uniNBaseAddress[offset >> 2];
 }
 
@@ -320,15 +285,10 @@ UInt32 AppleU3::readUniNReg(UInt32 offset)
 void AppleU3::writeUniNReg(UInt32 offset, UInt32 data)
 {
     uniNBaseAddress[offset >> 2] = data;
-	
-	// [3313953] reads to U3 Pwr Mgmt registers can trigger a clock synchronization
-	// boundary crossing problem.  To prevent this a read of the version register
-	// is sufficient.
-	if ((offset >= kU3PMClockControl) && (offset <= kU3PMSMax))
-	    (void) readUniNReg(kUniNVersion);
+
 
     OSSynchronizeIO();
-	
+
 	return;
 }
 
@@ -400,7 +360,8 @@ void AppleU3::uniNSetPowerState (UInt32 state)
 	} else if (state == kUniNIdle2) {
 		IOLog ("AppleU3: Idle 2 state not supported\n");
 	} else if (state == kUniNSleep) {		// sleep
-		spu->callPlatformFunction (symSetSPUSleep, false, (void *)false, (void *)0, (void *)0, (void *)0);
+		if (spu)	// Only call spu if present [3448210]
+			spu->callPlatformFunction (symSetSPUSleep, false, (void *)false, (void *)0, (void *)0, (void *)0);
 			
 		// Set the sleeping state for HWInit.  This tells OF we were sleeping
 		safeWriteRegUInt32(kUniNHWInitState, ~0UL, kUniNHWInitStateSleeping);
@@ -412,16 +373,6 @@ void AppleU3::uniNSetPowerState (UInt32 state)
 		// Set HyperTransport back to default state
 		setHTLinkFrequency (0);		// Set U3 end link frequency to 200 MHz
 
-		// golem hack!
-		if (golem) {
-			// Configure golem for 200MHz
-			data = golem->configRead32 (0xcc);
-			data = (data & 0xFFFFF0FF);
-			golem->configWrite32 (0xcc, data);
-			data = golem->configRead32 (0xd0);
-			data = (data & 0xFFFFF0FF);
-			golem->configWrite32 (0xd0, data);
-		}
 			
 			// Set K2 end link frequency to 200MHz
 		if (k2)
@@ -432,13 +383,7 @@ void AppleU3::uniNSetPowerState (UInt32 state)
 		// Set K2 end link width 8-bit - not required as K2 only runs 8 bit
 		// if (k2) k2->callPlatformFunction (symSetHTLinkWidth, false, (void *)0, (void *)0, (void *)0, (void *)0);
 
-		// golem hack!
-		if (golem) {
-			// Configure golem for 8-bit
-			data = golem->configRead32 (0xc4);
-			data = (data & 0x88FFFFFF);
-			golem->configWrite32 (0xc4, data);
-		}
+
 	}
 	
 	return;
@@ -601,23 +546,27 @@ IOPCIDevice* AppleU3::findNubForPHandle( UInt32 pHandleValue )
 void AppleU3::prepareForSleep ( void )
 {
 	IOService *service;
-	static bool noGolem;
+	static bool noGolem, noSPU;
 	
 	// Find the K2 driver
 	if (!k2)
 		k2 = waitForService(serviceMatching("KeyLargo"));
 	
-	// Find the SPU driver
-	if (!spu) {
-		service = waitForService(resourceMatching("AppleSPU"));
-		if (service) 
-			spu = OSDynamicCast (IOService, service->getProperty("AppleSPU"));
+	// Find the SPU driver, if present
+	if (!(spu || noSPU)) {
+		if (fromPath("/spu", gIODTPlane))		// Does /spu exist [3448210]?
+			spu = waitForService(serviceMatching("AppleSPU"));	// Yes, wait for associated driver
+
+		noSPU = (spu == NULL);				// If node or driver not found, set noSPU true so we don't keep looking for it
 	}
 	
 	// Find the PMU device
-	if (!pmu) 
-		pmu = waitForService(serviceMatching("ApplePMU"));	
-	
+	if (!pmu) {
+		service = waitForService(resourceMatching("IOPMU"));
+		if (service) 
+			pmu = OSDynamicCast (IOService, service->getProperty("IOPMU"));
+	}
+		
 	// Find golem, if present
 	if (!(golem || noGolem)) {
 		golem = OSDynamicCast (IOPCIDevice, provider->fromPath("/ht@0,F2000000/pci@1", gIODTPlane));
@@ -635,26 +584,11 @@ void AppleU3::prepareForSleep ( void )
 	return;
 }
 
-/*
- * getHTLinkFrequency - return the current HyperTransport link frequency.
- * The result is in bits defined for the link frequency, not in absolute
- * frequency.  The result may be interpret as follows:
- *
- *		0000b			 200MHz
- *		0001b			 300MHz
- *		0010b			 400MHz
- *		0011b			 500MHz
- *		0100b			 600MHz
- *		0101b			 800MHz
- *		0110b			1000MHz
- *		0111b - 1110b	Reserved
- *		1111b			Vendor Specific
- */
+
 bool AppleU3::getHTLinkFrequency (UInt32 *freqResult)
 {
 	UInt32			freq;
 
-	freq = safeReadRegUInt32 (0x70100);
 	freq = safeReadRegUInt32 (kU3HTLinkFreqRegister);
 	freq = (freq >> 8) & 0xF;
 	*freqResult = freq;
@@ -675,17 +609,7 @@ bool AppleU3::setHTLinkFrequency (UInt32 newFreq)
 	return true;
 }
 
-/*
- * getHTLinkWidth - return the current HyperTransport link in/out width.
- * The results (in and out) may be interpret as follows:
- *
- *		000b	 8-bit
- *		001b	 16-bit
- *		011b	 32-bit
- *		100b	 2-bit
- *		101b	 4-bit
- *		111b	 disconnected
- */
+
 bool AppleU3::getHTLinkWidth (UInt32 *linkOutWidthResult, UInt32 *linkInWidthResult)
 {
 	UInt32			width;
@@ -715,12 +639,325 @@ bool AppleU3::setHTLinkWidth (UInt32 newLinkOutWidth, UInt32 newLinkInWidth)
 // **********************************************************************************
 void AppleU3::u3APIPhyDisableProcessor1 ( void ) 
 {
-	/*
-	 * In a two processor system where the second processor is unused, OF has left
-	 * that processor in a spin loop executing out of ROM.  This steals bus cycles so
-	 * we workaround it by taking the processor off the bus [3249029].  This also fixes
-	 * a hang when boot-args cpus=1 is set [3273619]
-	 */
+
+
 	safeWriteRegUInt32 (kU3APIPhyConfigRegister1, ~0UL, 0);
 	return;
 }
+
+// **********************************************************************************
+// installChipFaultHandler
+//
+// The U3 twins (lite/heavy) have a chip fault signal (CHP_FAULT_N) that the system
+// implementors can attach to a GPIO.  Chip fault is asserted in response to an
+// ApplePI exception and can be masked with the Chip Fault Mask Register.
+//
+// This routine will be called from AppleU3::start() if there is a
+// "platform-chip-fault" property in the u3 device tree node.  It will create the
+// appropriate platform function symbols and install a callback function to handle
+// chip fault events.
+//
+// ECC memory errors are delivered to the system using this mechanism.
+//
+// **********************************************************************************
+IOReturn AppleU3::installChipFaultHandler ( IOService * provider )
+{
+	const OSData	*provider_phandle;
+	char			stringBuf[256];
+	UInt32			pHandle;
+
+	symChipFaultFunc = symPFIntRegister = symPFIntEnable = symPFIntDisable = NULL;
+
+	// We should have a chip fault signal on U3 lite and heavy
+	if (provider->getProperty(kChipFaultFuncName) == NULL)
+	{
+		if (IS_U3_TWINS(uniNVersion))
+			IOLog("AppleU3: WARNING: platform-chip-fault expected, but not found\n");
+	
+		return kIOReturnUnsupported;
+	}
+
+	if ((provider_phandle = OSDynamicCast( OSData, provider->getProperty("AAPL,phandle") )) == NULL)
+		return kIOReturnNotFound;
+
+	symPFIntRegister	= OSSymbol::withCString(kIOPFInterruptRegister);
+	symPFIntEnable		= OSSymbol::withCString(kIOPFInterruptEnable);
+	symPFIntDisable		= OSSymbol::withCString(kIOPFInterruptDisable);
+
+	// construct a function symbol of the form "platform-chip-fault-ff001122"
+	pHandle = *((UInt32 *)provider_phandle->getBytesNoCopy());
+	sprintf(stringBuf, "%s-%08lx", kChipFaultFuncName, pHandle);
+	symChipFaultFunc = OSSymbol::withCString(stringBuf); 
+
+	// Mask all chip fault sources.  Bits that we want unmasked will be handled separately.
+	safeWriteRegUInt32 ( kU3ChipFaultMaskRegister, ~0UL, 0 );
+
+	// register for notifications
+	return callPlatformFunction(symChipFaultFunc, TRUE,
+			(void *) AppleU3::sHandleChipFault, this, NULL, (void *) symPFIntRegister);
+}
+
+// **********************************************************************************
+// sHandleChipFault
+//
+// C-style static callback for chip fault interrupt.
+//
+// **********************************************************************************
+
+/* static */	/* executing on system workloop */
+void AppleU3::sHandleChipFault( void * vSelf, void * vRefCon, void * /* NULL */, void * /* unused */ )
+{
+	UInt32 apiexcp;
+
+	AppleU3 * me = OSDynamicCast( AppleU3, (OSMetaClassBase *) vSelf );
+
+	// kprintf( "AppleU3::sHandleChipFault\n" );
+
+	if (!me) return;
+
+	// read the APIEXCP register to find out the source of this event.  the read operation
+	// causes the faults to be cleared.
+	apiexcp = me->safeReadRegUInt32( kU3APIExceptionRegister );
+
+	// kprintf( "AppleU3 Got Chip Fault! APIEXCP = %08lX\n", apiexcp );
+
+	// Catch DART exceptions
+	if (apiexcp & kU3API_DARTExcp)
+	{
+		char errstr[128];
+		UInt32 dartexcp = me->safeReadRegUInt32( kU3DARTExceptionRegister );
+
+		sprintf( errstr, "DART %s%s%s %s logical page 0x%05lX\n",
+			( dartexcp & kU3DARTExcpXBEMask ) ? "out-of-bounds exception: " : "",
+			( dartexcp & kU3DARTExcpXEEMask ) ? "entry exception: " : "",
+			( dartexcp & kU3DARTExcpRQSRCMask ) ? "HyperTransport" : "PCI0",
+			( dartexcp & kU3DARTExcpRQOPMask ) ? "write" : "read",
+			( dartexcp & kU3DARTExcpLogAdrsMask ) >> kU3DARTExcpLogAdrsShift );
+
+		// kaboom!
+		panic( errstr );
+	}
+
+	// if this is U3 Heavy, Check for ECC error
+	if (IS_U3_HEAVY(me->uniNVersion) &&
+	    (apiexcp & (kU3API_ECC_UE_H | kU3API_ECC_UE_L | kU3API_ECC_CE_H | kU3API_ECC_CE_L)))
+	{
+		UInt32 mear, mesr, rank, dimmloc;
+
+		// interrogate U3 ECC registers
+		mear = me->safeReadRegUInt32( kU3MemErrorAddressRegister );
+		mesr = me->safeReadRegUInt32( kU3MemErrorSyndromeRegister );
+
+		// get the dimm slot index
+		rank = (mear & kU3MEAR_RNK_A_mask) >> kU3MEAR_RNK_A_shift;
+
+		// Check for an uncorrectable error
+		if (apiexcp & (kU3API_ECC_UE_H | kU3API_ECC_UE_L))
+		{
+			dimmloc = rank - (rank%2) + (apiexcp & kU3API_ECC_UE_H ? 0 : 1);
+
+			panic("Uncorrectable parity error detected in %s (MEAR=0x%08lX MESR=0x%08lX)\n",
+					me->dimmErrors[dimmloc].slotName, mear, mesr);
+		}
+
+		// must be a correctable error.  increment the count for this DIMM
+		else // if (apiexcp & (kU3API_ECC_CE_H | kU3API_ECC_CE_L))
+		{
+			dimmloc = rank - (rank%2) + (apiexcp & kU3API_ECC_CE_H ? 0 : 1);
+
+			//kprintf("AppleU3 got correctable error dimm %u\n", dimmloc);
+
+			IOSimpleLockLock( me->dimmLock );
+			me->dimmErrors[dimmloc].count++;
+			IOSimpleLockUnlock( me->dimmLock );
+
+			// schedule a notification thread callout (if not already scheduled)
+			if (thread_call_is_delayed( me->eccErrorCallout, NULL ) == FALSE)
+			{
+				// kprintf("AppleU3 scheduling notifier thread\n");
+
+				AbsoluteTime deadline;
+				clock_interval_to_deadline( kU3ECCNotificationIntervalMS, kMillisecondScale, &deadline );
+				thread_call_enter1_delayed( me->eccErrorCallout, vRefCon, deadline);
+			}
+		}
+	}
+}
+
+// **********************************************************************************
+// sDispatchECCNotifier
+//
+// C-style callback method for periodic ECC notification.
+//
+// **********************************************************************************
+
+/* static */
+void AppleU3::sDispatchECCNotifier( void *self, void *refcon )
+{
+	AppleU3 * me = OSDynamicCast( AppleU3, (OSMetaClassBase *) self );
+
+	//kprintf("AppleU3::sDispatchECCNotifier\n");
+
+	if (me) me->eccNotifier( refcon );
+}
+
+// **********************************************************************************
+// eccNotifier
+//
+// C++-style callback method for periodic ECC notification.
+//
+// **********************************************************************************
+
+void AppleU3::eccNotifier( void * refcon )
+{
+	UInt32 slot;
+	UInt32 * dimmErrorCounts;
+	u3_parity_error_msg_t msg;
+
+	// kprintf("AppleU3::eccNotifier\n");
+
+	// allocate memory for a copy of the dimm error counts
+	dimmErrorCounts = (UInt32 *) IOMalloc( dimmCount * sizeof(UInt32) );
+
+	if (!dimmErrorCounts)
+	{
+		IOLog("ECC Notifier -- failed to allocate memory!\n");
+		return;
+	}
+
+	// get a copy of the dimm error counts, and zero out the live array
+	IOSimpleLockLock( dimmLock );
+
+	for (slot=0; slot<dimmCount; slot++)
+	{
+		dimmErrorCounts[slot] = dimmErrors[slot].count;
+		dimmErrors[slot].count = 0;
+	}
+
+	IOSimpleLockUnlock( dimmLock );
+
+	// post a notification for each slot that's encountered errors
+	for (slot=0; slot<dimmCount; slot++)
+	{
+		if (dimmErrorCounts[slot] > 0)
+		{
+			// write to system log
+			IOLog("WARNING: %lu parity errors corrected in %s\n",
+					dimmErrorCounts[slot], dimmErrors[slot].slotName);
+
+			// send a notification to interested clients
+			msg.version = 0x1;
+			msg.slotIndex = slot;
+			strncpy(msg.slotName, dimmErrors[slot].slotName, 32);
+			msg.count = dimmErrorCounts[slot];
+		
+			messageClients( kIOPlatformMessageParityError,
+					(void *) &msg, sizeof(u3_parity_error_msg_t) );
+		}
+	}
+
+	IOFree( dimmErrorCounts, dimmCount * sizeof(UInt32) );
+}
+
+// **********************************************************************************
+// setupECC
+//
+// If we are on U3 heavy and ECC has been enabled by HWInit, unmask the chip fault
+// sources for ECC errors.
+//
+// **********************************************************************************
+
+void AppleU3::setupECC( void )
+{
+	IORegistryEntry * memoryNode;
+	const OSData * slotNamesData;
+	const char * slotNames;
+	UInt32 bits, i, slotBitField;
+
+	// If it's not U3 Heavy, we don't have ECC
+	if (!IS_U3_HEAVY(uniNVersion)) return;
+
+	// ECC is initialized and enabled by HWInit if possible.  Check if it is turned on.
+	if ( ! ( kU3MCCR_ECC_EN & safeReadRegUInt32( kU3MemCheckCtrlRegister ) )) return;
+
+	// grab the DIMM slot names from the device tree, they are in the /memory node under the
+	// slot-names property.  The first word of the property is a bit field with one bit set
+	// for each available slot.  A null-terminated c-style ascsii string follows, one for each
+	// slot.
+	if ((memoryNode = fromPath("/memory", gIODTPlane, 0, 0, 0)) == NULL) return;
+	slotNamesData = OSDynamicCast(OSData, memoryNode->getProperty("slot-names"));
+	memoryNode->release();
+	if (!slotNamesData) return;
+
+	slotNames = (const char *) slotNamesData->getBytesNoCopy();
+	slotBitField = *(const UInt32 *) slotNames;	// grab the bitfield word
+	slotNames += sizeof(UInt32);	// advance the pointer to the first ascii name
+
+	// count the number of slots we have
+	dimmCount = 0;
+	for (i=0; i<32; i++)
+	{
+		if (!(slotBitField & (1 << i))) break;
+		dimmCount++;
+	}
+
+	if (dimmCount > kU3MaxDIMMSlots) return;
+
+	// kprintf("AppleU3::setupECC dimmCount is %u\n", dimmCount);
+
+	// allocate a thread callout so we can service chip faults without worrying about
+	// blocking someone's workloop
+	eccErrorCallout = thread_call_allocate((thread_call_func_t) AppleU3::sDispatchECCNotifier,
+													(thread_call_param_t) this);
+	if (!eccErrorCallout) return;
+
+	// allocate and initialize the lock that protects the error counts
+	if ((dimmLock = IOSimpleLockAlloc()) == NULL) return;
+	IOSimpleLockInit( dimmLock );
+
+	// allocate an array to hold all the dimm error counts
+	dimmErrors = (u3_parity_error_record_t *) IOMalloc( sizeof(u3_parity_error_record_t) * dimmCount );
+	if (!dimmErrors)
+	{
+		IOSimpleLockFree( dimmLock );
+		dimmLock = NULL;
+		return;
+	}
+
+	// copy slot names into the array -- no need to lock this yet, since other threads won't
+	// try to modify it until we've unmasked the error bits below
+	for (i=0; i<dimmCount; i++)
+	{
+		strncpy( dimmErrors[i].slotName, slotNames, 31 );	// copy the slot name
+		dimmErrors[i].slotName[31] = '\0';	// guarantee a terminating null
+		dimmErrors[i].count = 0;	// zero the error count
+
+		slotNames += strlen(slotNames) + 1;	// advance to the next dimm name
+	}
+
+	IOLog( "Enabling ECC Error Notifications\n" );
+
+	// flag that this is an ecc supported memory controller
+	setProperty("ecc-supported", "true" );
+
+	// Clear the mask bits in the MCCR to enable error propogation
+	bits = kU3MCCR_ECC_UE_MASK_H | kU3MCCR_ECC_CE_MASK_H | kU3MCCR_ECC_UE_MASK_L | kU3MCCR_ECC_CE_MASK_L;
+	safeWriteRegUInt32( kU3MemCheckCtrlRegister, bits, ~bits );
+
+	// Set the mask bits in the CFMR to enable chip fault generation
+	bits = kU3API_ECC_UE_H | kU3API_ECC_CE_H | kU3API_ECC_UE_L | kU3API_ECC_CE_L;
+	safeWriteRegUInt32( kU3ChipFaultMaskRegister, bits, bits );
+}
+
+// **********************************************************************************
+// setupDARTExcp
+//
+// This if called to enable DART exception notification via U3Twins' chip fault.
+//
+// **********************************************************************************
+
+void AppleU3::setupDARTExcp( void )
+{
+	// Set the mask bit in the CFMR to enable chip fault generation
+	safeWriteRegUInt32( kU3ChipFaultMaskRegister, kU3API_DARTExcp, kU3API_DARTExcp );
+}
